#!/usr/bin/env python
# -*- coding: utf-8 -*-
#*****************************************************************************
#       Copyright (C) 2006 Serge Weinstock <serge.weinstock@ntlworld.com>
#  Distributed under the terms of the BSD License.  The full license is in
#  the file COPYING, distributed as part of this software.
#*****************************************************************************
"""
The batch conversion dialog
"""
import webshots
from batchconversion import BatchConvertDlg
from qt import Qt, QString, QCheckBox, PYSIGNAL, QCursor, QCustomEvent, QColorGroup
from kdecore import KApplication, i18n, KURL, KTempFile
from kdeui import KListViewItem
from kfile import KFileDialog, KFile
from kdeutils import DirLister, getCustomEventID
from kiojob import Job, runJob, EndJobEvent
from config import MAX_THUMB_SIZE

FORBIDDEN_CHAR_IN_PATH = '\/:*?"<>|'

class URLListItem(KListViewItem):
    """
    A list view item which display an URL
    """
    STATUS_NOTDONE = 0
    STATUS_OK = 1
    STATUS_ERROR = 2
    def __init__(self, url, parent):
        """
        Constructor
        @param[in] url the item URL
        @param[in] parent the parent listview item
        """
        KListViewItem.__init__(self, parent, "")
        self.url = KURL(url)
        self.setText(0, self.url.fileName())
        self.setText(1, self.url.upURL().prettyURL())
        self.status = URLListItem.STATUS_NOTDONE
        self.urls = set()
    def setStatus(self, success, message):
        '''
        Change the status of this element
        @param[in] success True if the conversion was done successfully
        @param message the message to display
        '''
        if success:
            self.status = URLListItem.STATUS_OK
        else:
            self.status = URLListItem.STATUS_ERROR
        self.setText(2, message)
    def paintCell(self, painter, colorGroup, column, width, alignment):
        """
        Paint a cell
        It paint the text
        - in the normal color if the tranformation was successfull
        - in red if the tranformation was unsuccessfull and the item is checked
        - in dark magenta if the tranformation was unsuccessfull and the item is not checked

        @param[in] painter  the QPainter open on the relevant paint device
        @param[in] colorGroup  the color group to use
        @param[in] column  the logical column number within the item that is to be painted
        @param[in] width the cell width
        @param[in] alignment the cell alignment
        """
        _cg = QColorGroup(colorGroup)
        c = _cg.text();
        if self.status == URLListItem.STATUS_ERROR:
            _cg.setColor(QColorGroup.Text, Qt.red)
        elif self.status == URLListItem.STATUS_OK:
            _cg.setColor(QColorGroup.Text, Qt.darkBlue)
        KListViewItem.paintCell(self, painter, _cg, column, width, alignment );
        _cg.setColor(QColorGroup.Text, c)

class SelectDirDialog(KFileDialog):
    """
    A select directory dialog with an option for recursing through the directory
    """
    def __init__(self, startDir, recurse, parent):
        '''
        Constructor
        @param[in] startDir the initial directory
        @param[in] recurse if True, check the "recurse" checkbox
        @param[in] parent the dialog parent
        '''
        self.recurse = QCheckBox(i18n("Recurse directories"), None, "recurseDir")
        # there is a problem when the protocol is not file://, the name of the directory is displayed
        # in the file box. A little trick for "hiding" that
        startDir = QString(startDir)
        if startDir[-2:] == "./":
            startDir = startDir[:-1]
        elif startDir[-1:] == "/":
            startDir += "."
        elif startDir[-2:] != "/.":
            startDir += "/."
        KFileDialog.__init__(self, startDir, QString.null, parent, "select_dir", True, self.recurse)
        self.recurse.setOn(bool(recurse))
        self.setMode(KFile.Directory)
    @staticmethod
    def run(initialDir, recurse, parent):
        '''
        Launch the dialog
        @param[in] initialDir the initial directory
        @param[in] recurse if True, check the "recurse" checkbox
        @param[in] parent the dialog parent
        @return (selected url, recurse directory)
        '''
        dlg = SelectDirDialog(initialDir, recurse, parent)
        dlg.exec_loop()
        return (dlg.selectedURL(), dlg.recurse.isOn())

class BatchEvent(QCustomEvent):
    '''
    An event for updating the status of an URL in the file list
    '''
    ID = getCustomEventID()
    def __init__(self, li, status, message):
        '''
        Constructor
        @param[in] li the processed list item
        @param[in] status True if the conversion was successfull
        @param[in] message a message to display
        '''
        QCustomEvent.__init__(self, self.ID)
        self.li = li
        self.status = status
        self.message = message

class BatchJob(Job):
    def __init__(self, output_folder, lis, replace_mode, filterSizes, owner):
        '''
        The job which converts the webshots files
        @param[in] output_folder the output folder
        @param[in] lis the listview items to process
        @param[in] replace_mode how to handle files with the same name
        @param[in] filterSizes which image size to keep
        @param[in] owner the widget to notify of progress
        '''
        Job.__init__(self, owner)
        if output_folder[:-1] != "/": output_folder = unicode(output_folder) + u"/"
        self.output_folder = KURL(output_folder)
        self.lis = lis
        self.replace_mode = replace_mode
        self.filterSizes = filterSizes
        self.owner = owner
        self.lv_sort = None # temporary variable used for keeping the lisview sort settings
    def sendBatchStatus(self, url, status, message):
        '''
        Set the text of the progress dialog
        @param[in] text the text to display
        '''
        if self.dialog:
            KApplication.kApplication().processEvents()
            KApplication.kApplication().postEvent(self.owner, BatchEvent(url, status, message))
    def run(self):
        '''
        The job main routine
        '''
        idx = 0
        for li in self.lis:
            url = li.url
            self.checkCancelled()
            yield self.setProgress(steps = idx, totalsteps = len(self.lis))
            # if the file is not local. download it
            yield self.kio_download(url)
            if self.kio.error:
                self.sendBatchStatus(li, False, i18n("Unable to access the file! (%1)").arg(self.kio.errorString))
                continue
            # let's process it
            try:
                target = self.kio.res
                self.checkCancelled()
                f = open(unicode(target), 'rb')
                try:
                    # let's try first a WB[1DZ] file
                    pixmap = webshots.WB1.read(url, f, buildPixmap = False)
                    if pixmap:
                        pixmaps = [ pixmap ]
                    else:
                        # let's try a WBC file
                        title, pixmaps = webshots.WBC.read(url, f, buildPixmap = False) #@UnusedVariable
                        if not pixmaps:
                            self.sendBatchStatus(li, False, i18n("Invalid file type!"))
                            continue
                finally:
                    del f
            finally:
                self.kio_removeTempFile(target)
            self.checkCancelled()
            # now let's save the files
            npc = 0
            for pixmap in pixmaps:
                # right size?
                if self.filterSizes[0]:
                    w, h = pixmap.getJPEGSize() #@UnusedVariable
                    if w <= 0:
                        self.sendBatchStatus(li, False, i18n("Invalid file format!"))
                        break # for pixmap in pixmaps
                    if w <= MAX_THUMB_SIZE:
                        if not self.filterSizes[1][MAX_THUMB_SIZE]:
                            continue # for pixmap in pixmaps
                    elif not self.filterSizes[1].get(w, False):
                        continue # for pixmap in pixmaps
                # build the output file name
                fn = pixmap.info.get(webshots.K_TITLE, "image")
                ext = ".jpg"
                for c in FORBIDDEN_CHAR_IN_PATH:
                    fn = fn.replace(c, "_")
                durl = KURL(self.output_folder, QString(fn+ext))
                if self.replace_mode != BatchConvertDlgImpl.OVERWRITE_REPLACE:
                    # handle duplicates
                    yield self.kio_exists(durl)
                    self.checkCancelled()
                    if self.kio.res:
                        if self.replace_mode == BatchConvertDlgImpl.OVERWRITE_SKIP:
                            continue # for pixmap in pixmaps
                        if self.replace_mode == BatchConvertDlgImpl.OVERWRITE_RENAME:
                            for i in xrange(1000):
                                durl = KURL(self.output_folder, QString(fn + "_%03d" % i + ext))
                                yield self.kio_exists(durl)
                                self.checkCancelled()
                                if not self.kio.res:
                                    break # for i loop
                            else:
                                self.sendBatchStatus(li, False, i18n("Unable to rename the file!"))
                                break # for pixmap in pixmaps
                # now save it
                if durl.isLocalFile():
                    ### local file
                    of = open(unicode(durl.path()), "wb")
                    try:
                        pixmap.jpeg.tofile(of)
                    finally:
                        of.close()
                    npc += 1
                    self.checkCancelled()
                    continue
                ### remote file
                tmpFile = KTempFile()
                try:
                    tmpFile.setAutoDelete(True);
                    of = open(unicode(tmpFile.name()), "wb")
                    pixmap.jpeg.tofile(of)
                    of.close()
                    yield self.kio_upload(tmpFile.name(), durl)
                    self.checkCancelled()
                    if self.kio.error:
                        self.sendBatchStatus(li, False, i18n("Could not save remote file! (%1)").arg(self.kio.errorString))
                        break
                    npc += 1
                    del tmpFile
                except:
                    del tmpFile
                    raise
            # finished the processing of one webshot file
            # let's update the file list
            if npc:
                self.sendBatchStatus(li, True, i18n("Success."))
            else:
                self.sendBatchStatus(li, True, i18n("No image converted"))
            idx += 1
        return

class BatchConvertDlgImpl(BatchConvertDlg):
    OVERWRITE_REPLACE = 0
    OVERWRITE_SKIP = 1
    OVERWRITE_RENAME = 2
    def __init__(self,parent = None,name = None,modal = True,fl = 0):
        BatchConvertDlg.__init__(self,parent,name,modal,fl)
        self.urls = set()    # a set of URL string for accelerating duplicate search
        self.maincfg = KApplication.kApplication().cfg.mainframe # the mainframe config
        self.cfg = KApplication.kApplication().cfg.batch # this dialog config
        # set up the dialog
        self.url_outputFolder.setMode(KFile.Directory)
        self.url_outputFolder.setURL(self.maincfg.lastSaveDir)
        o = self.cfg.overwriteMode
        if o == self.OVERWRITE_SKIP:
            self.rbtn_skip.setOn(True)
        elif o == self.OVERWRITE_RENAME:
            self.rbtn_rename.setOn(True)
        else:
            self.rbtn_overwrite.setOn(True)
        filterSizes = self.cfg.filterSizes
        self.btnGroup_sizes.setChecked(filterSizes[0])
        self.chk_SizeThumbs.setOn(filterSizes[1][MAX_THUMB_SIZE])
        self.chk_Size800.setOn(filterSizes[1][800])
        self.chk_Size1600.setOn(filterSizes[1][1600])
        self.chk_Size1920.setOn(filterSizes[1][1920])
        self.updateUI()
        self.dirLister = DirLister(KApplication.kApplication().getFileFilter())
        self.connect(self.dirLister,PYSIGNAL("newItems"), self.newItems)
        self.connect(self.dirLister,PYSIGNAL("completed"), self.completed)
        sz = self.cfg.size
        if not sz.isEmpty():
            self.resize(sz)

    def updateUI(self):
        '''
        Update the UI
        '''
        url = KURL(self.url_outputFolder.url())
        hasFiles = self.lstvw_files.childCount() != 0
        hasSel = len(self.lstvw_files.selectedItems()) != 0
        self.btn_clear.setEnabled(hasFiles)
        self.btn_remove.setEnabled(hasSel)
        self.btn_Ok.setEnabled(
               ((not url.isEmpty()) and url.isValid()) and
               (hasFiles)
               )
        if not self.urls:
            self.lbl_nFiles.setText(i18n("No file"))
        elif len(self.urls) == 1:
            self.lbl_nFiles.setText(i18n("1 file"))
        else:
            self.lbl_nFiles.setText(i18n("%1 files").arg(len(self.urls)))

    def newItems(self, items):
        '''
        Slot for kdeurils.DirLister.newItems
        @param[in] items the list of found items
        '''
        for item in items:
            if item.isFile():
                # avoid duplicate entries
                url = KURL(item.url()) # make a copy as the returned URL is a temporary instance
                if unicode(url.prettyURL()) not in self.urls:
                    self.urls.add(unicode(url.prettyURL()))
                    URLListItem(url, self.lstvw_files)
    def completed(self, success):
        '''
        Slot for kdeurils.DirLister.completed
        @param[in] success true if the listing was successfull
        '''
        KApplication.kApplication().restoreOverrideCursor()
        self.setDisabled(False)
        if success:
            self.maincfg.lastOpenDir = self.dirLister.url.prettyURL()
            self.cfg.recursive = self.dirLister.recursive
        # restore the sorting
        self.lstvw_files.setSorting(*self.lv_sort)
        self.updateUI()
    def onAddFiles(self):
        '''
        Add one or several files to the list of files to process
        '''
        urls = KApplication.kApplication().selectFiles(self)
        if not urls:
            return
        for url in urls:
            # avoid duplicate entries
            if unicode(url.prettyURL()) not in self.urls:
                self.urls.add(unicode(url.prettyURL()))
                URLListItem(url, self.lstvw_files)
        self.maincfg.lastOpenDir = url.upURL().prettyURL()
        self.updateUI()

    def onAddFolder(self):
        '''
        Add the contents of a folde to the list of files to process
        '''
        url, recursive = SelectDirDialog.run(self.maincfg.lastOpenDir, self.cfg.recursive, self)
        if url.isEmpty():
            return
        KApplication.kApplication().setOverrideCursor(QCursor.waitCursor)
        self.setDisabled(True)
        # disable sorting in order to speed up the insertion of many items
        self.lv_sort = (self.lstvw_files.sortColumn(), self.lstvw_files.sortOrder())
        self.lstvw_files.setSorting(-1)
        self.dirLister.openURL(url, recursive)
    def onRemove(self):
        '''
        Remove the selected items from the list of files
        '''
        for i in self.lstvw_files.selectedItems():
            try:
                self.urls.remove(unicode(i.url.prettyURL()))
            except KeyError:
                pass
            self.lstvw_files.takeItem(i)
        self.updateUI()
    def onClearAll(self):
        '''
        Remove all items from the list of files
        '''
        self.urls = set()
        self.lstvw_files.clear()
        self.updateUI()
    def onOutputFolderChanged(self, url): #@UnusedVariable
        '''
        Called when the url for the output folder changes
        @param[in] url the new URL
        '''
        self.updateUI()
    def accept(self):
        '''
        Called when the user clicks on OK
        '''
        outputDir = self.url_outputFolder.url()
        if self.rbtn_skip.isOn():
            o = self.OVERWRITE_SKIP
        elif self.rbtn_rename.isOn():
            o = self.OVERWRITE_RENAME
        else:
            o = self.OVERWRITE_REPLACE
        lis = []
        li = self.lstvw_files.firstChild()
        while li:
            lis.append(li)
            li = li.nextSibling()
        filterSizes = [True, {}]
        filterSizes[0] = self.btnGroup_sizes.isChecked()
        filterSizes[1][MAX_THUMB_SIZE] = self.chk_SizeThumbs.isOn()
        filterSizes[1][800] = self.chk_Size800.isOn()
        filterSizes[1][1600] = self.chk_Size1600.isOn()
        filterSizes[1][1920] = self.chk_Size1920.isOn()
        job = BatchJob(outputDir, lis, o, filterSizes, self)
        success, results = runJob(job, self, self, i18n("Batch conversion"), i18n("Converting files"), i18n("&Cancel"), True) #@UnusedVariable
        self.cfg.overwriteMode = o
        self.cfg.filterSizes = filterSizes
        self.maincfg.lastSaveDir = outputDir
    def customEvent(self, event):
        """
        This is the event handler for custom events
        @param[in] event the QCustomEvent event
        """
        if event.type() == BatchEvent.ID:
            # let's update the status of the item
            self.lstvw_files.ensureItemVisible(event.li)
            event.li.setStatus(event.status, event.message)
        elif event.type() == EndJobEvent.ID:
            self.btn_Cancel.setText(i18n("&Close"))
    def hideEvent (self, event):
        """
        Called when the window is hidden
        @param[in] event a QCloseEvent
        """
        self.cfg.size = self.size()
        BatchConvertDlg.hideEvent(self, event)
