#!/usr/bin/env python
# -*- coding: utf-8 -*-
#*****************************************************************************
#       Copyright (C) 2006 Serge Weinstock <serge.weinstock@ntlworld.com>
#  Distributed under the terms of the BSD License.  The full license is in
#  the file COPYING, distributed as part of this software.
#*****************************************************************************
import sys, release
from qt import QString
from kdecore import KAboutData, KCmdLineArgs, KUniqueApplication, i18n
from kfile import KFileDialog
from mainframe import MainFrame
from kdeappiface import kUWCIface
from config import Config

def I18N_NOOP(str):
    """Workaround the fact that PyKDE doesn't provide a I18N_NOOP"""
    return str

OPTIONS = [
    ("vv", I18N_NOOP("be verbose")),
    ("!+URLs", I18N_NOOP("the urls to process")),
    ]

__verbose = False

class UWCApp(KUniqueApplication):
    """
    The application class
    """
    FILTER_SINGLE_ARCHIVE = 0x01
    FILTER_COLLECTION_ARCHIVE= 0x02
    FILTER_ALL = FILTER_SINGLE_ARCHIVE | FILTER_COLLECTION_ARCHIVE

    def __init__(self):
        """
        Constructor
        """
        KUniqueApplication.__init__(self, True, True, False    )
        self.cfg = Config()
        self.iface = None

    def setup(self, options):
        '''
        Setup the application
        @param[in] options the KCmdLineArgs read from the command line
        '''
        # see if we are starting with session management
        if self.isRestored():
            if MainFrame.canBeRestored(1):
                self.mainWindow = MainFrame()
                self.mainWindow.restore(1, True)
                self.mainWindow.show()
            self.setMainWidget(self.mainWindow)
        else:
            # no session.. just start up normally
            # let's start the UI
            self.mainWindow = MainFrame()
            self.setMainWidget(self.mainWindow)
            self.mainWindow.show()
            # open any requested file
            for i in range(options.count()):
                self.mainWindow.openURL(options.url(i))
        # we don't need to gregister ourselves as dcop server as KUniqueApplication does it for us
        # but we still need to register our new dcop interface
        self.iface = kUWCIface()

    def getFileFilter(self, flags = FILTER_ALL):
        '''
        Return a filter for webshot files
        @param flags a combination of  FILTER_SINGLE_ARCHIVE and FILTER_COLLECTION_ARCHIVE according to what we want to select
        @return a the QString set with the right filter
        '''
        filter = QString()
        if (flags & UWCApp.FILTER_SINGLE_ARCHIVE):
            filter += "*.wbz *.WBZ *.wb1 *.WB1 *.wbd *.WBD"
        if (flags & UWCApp.FILTER_COLLECTION_ARCHIVE):
            if filter: filter += ' '
            filter += "*.wbc *.WBC *.wbp *.WBP"
        return filter

    def selectFiles(self, parent = None, flags = FILTER_ALL):
        '''
        Select a list of WebShots files
        @param parent the widget parent of the open file dialog
        @param flags a combination of  FILTER_SINGLE_ARCHIVE and FILTER_COLLECTION_ARCHIVE according to what we want to select
        @return a KURL.List of KURL
        '''
        filter = QString()
        if (flags & UWCApp.FILTER_ALL):
            filter = QString("*.wbc *.WBC *.wbp *.WBP *.wbz *.WBZ *.wb1 *.WB1 *.wbd *.WBD|")
            filter += i18n("All WebShots files")
            filter += " (*.wbc *.wbp *.wbz *.wb1 *.wbd)"
        if (flags & UWCApp.FILTER_SINGLE_ARCHIVE):
            if filter: filter += '\n'
            filter += "*.wbz *.WBZ *.wb1 *.WB1 *.wbd *.WBD|"
            filter += i18n("WebShots pictures")
            filter += " (*.wb1 *.wbd)"
        if (flags & UWCApp.FILTER_COLLECTION_ARCHIVE):
            if filter: filter += '\n'
            filter += "*.wbc *.WBC *.wbp *.WBP|"
            filter += i18n("WebShots archives")
            filter += " (*.wbz)"
        # there is a problem when the protocol is not file://, the name of the directory is displayed
        # in the file box. A little trick for "hiding" that
        startDir = self.cfg.mainframe.lastOpenDir
        if startDir[-2:] == "./":
            startDir = startDir[:-1]
        elif startDir[-1:] == "/":
            startDir += "."
        elif startDir[-2:] != "/.":
            startDir += "/."
        urls = KFileDialog.getOpenURLs(
            startDir,
            filter,
            parent,
            i18n("Please select WebShots file(s)")
            )
        if not urls:
            return None
        self.cfg.lastOpenDir = urls[0].upURL().prettyURL()
        return urls

def run(argv):
    """
    Application initialisation and main loop
    """
    # let's first set up the about data
    aboutdata = KAboutData(
        release.appname,                # app internal name
        I18N_NOOP(release.name),        # app name for display
        release.version,                # version string
        I18N_NOOP(release.description), # short description
        KAboutData.License_BSD,         # license
        release.copyright,              # copyright statement
        None,                           # additional text
        release.url,                    # homepage address
        release.report_bugs_url         # bug report email address
        )
    for author in release.authors.itervalues():
        aboutdata.addAuthor(*author)
    for credit in release.credits.itervalues(): #@UndefinedVariable
        aboutdata.addCredit(*credit)
    # now let's parse the options
    KCmdLineArgs.init(argv, aboutdata, False)
    KCmdLineArgs.addCmdLineOptions(OPTIONS)
    KUniqueApplication.addCmdLineOptions()
    options = KCmdLineArgs.parsedArgs()
    #
    if not KUniqueApplication.start():
        from dcop import DCOPClient
        from dcopext import DCOPApp
        print >> sys.stderr, i18n("%1 is already running!").arg(release.appname)
        if options.count():
           # Forward the command line args to the running instance
           client = DCOPClient()
           if client.attach():
               ref = DCOPApp ("kUWC", client)
               for i in range(options.count()):
                   ref.main.openURL(options.url(i).prettyURL())
        sys.exit(0)
    #
    options.verbose = options.isSet("vv")
    __verbose = options.verbose
    # setup the application instance
    app = UWCApp()
    app.setup(options)
    # let's go!
    return app.exec_loop()

def main(argv=None):
    try:
        if argv is None:
            argv = sys.argv
        return run(argv)
    except SystemExit:
        raise
    except Exception, err:
        if __verbose:
            from traceback import print_exc
            print_exc(file = sys.stderr)
        print >>sys.stderr, str(err)
        print >>sys.stderr, "for help use --help"
        return 2

if __name__ == "__main__":
    sys.exit(main())
