#!/usr/bin/env python
# -*- coding: utf-8 -*-
#*****************************************************************************
#       Copyright (C) 2006 Serge Weinstock <serge.weinstock@ntlworld.com>
#  Distributed under the terms of the BSD License.  The full license is in
#  the file COPYING, distributed as part of this software.
#*****************************************************************************
"""
A framework for running a lengthy job on a worker thread and for displaying a progress dialog
"""
from qt import QThread, QCustomEvent, SIGNAL
from kdecore import i18n
from kdeui import KProgressDialog
from kdeutils import getCustomEventID

CANCEL_TIMEOUT = 20*1000

class ProgressEvent(QCustomEvent):
    '''
    An event for updating the progress bar of the progress dialog
    '''
    ID = getCustomEventID()
    def __init__(self, thread, progress, total):
        '''
        Constructor
        @param[in] thread the sender thread
        @param[in] progress the number of steps done
        @param[in] total the total number of steps
        '''
        QCustomEvent.__init__(self, self.ID)
        self.thread = thread
        self.progress = progress
        self.total = total

class StatusEvent(QCustomEvent):
    '''
    An event for updating the status text of the progress dialog
    '''
    ID = getCustomEventID()
    def __init__(self, thread, text):
        '''
        Constructor
        @param[in] thread the sender thread
        @param[in] text the text to display
        '''
        QCustomEvent.__init__(self, self.ID)
        self.thread = thread
        self.text = text

class TaskFinishedEvent(QCustomEvent):
    '''
    An event for notifying of the end of the job
    '''
    ID = getCustomEventID()
    def __init__(self, thread, message = None, success = True, data = None):
        '''
        Constructor
        @param[in] thread the sender thread
        @param[in] message an error or informational message
        @param[in] success True if the operation was successfull
        @param[in] data the resulting daya
        '''
        QCustomEvent.__init__(self, self.ID)
        self.thread = thread
        self.message = message
        self.success = success
        self.data = data

class Job(QThread):
    """A worker thread for doing lengthy operations."""
    class CancelException(Exception):
        '''
        An exception thrwon when the job is cancelled
        '''
        pass
    def __init__(self, dialog = None):
        """
        Constructor.
        @param dialog The progress dialog
        """
        QThread.__init__(self)
        self.cancelled = False
        self.dialog = dialog

    def setDialog(self, dialog):
        '''
        Set the progress dialog
        @param dialog the progress dialog
        '''
        self.dialog = dialog

    def cancel(self):
        '''
        set the job cancel flag to True
        '''
        self.cancelled = True
    def isCancelled(self):
        '''
        return True if the job cancel flags has been set
        '''
        return self.cancelled
    def checkCancelled(self):
        '''
        check if the job cancel flag has been set to True.
        If so it raises a CancelException.
        '''
        if self.cancelled:
            raise Job.CancelException()
    def setProgress(self, progress, total):
        '''
        Update the progress bar
        @param[in] progress the current progress count
        @param[in] total the total number of steps
        '''
        if self.dialog:
            self.postEvent(self.dialog, ProgressEvent(self, progress, total))
    def setStatusText(self, text):
        '''
        Set the text of the progress dialog
        @param[in] text the text to display
        '''
        if self.dialog:
            self.postEvent(self.dialog, StatusEvent(self, text))
    def setComplete(self, message = None, success = True, data = None):
        '''
        Notify of the progress completion
        @param[in] message an error or informational message
        @param[in] success True if the operation was successfull
        @param[in] data the optional resulting data
        '''
        if self.dialog:
            self.postEvent(self.dialog, TaskFinishedEvent(self, message, success, data))
    def run(self):
        '''
        The thread main function
        '''
        try:
            success, message, data = self.runJob()
        except Job.CancelException:
            success = False
            message = i18n("Cancelled")
            data = None
        except:
            from traceback import print_exception
            from StringIO import StringIO
            import sys
            exc_info = sys.exc_info()
            out = StringIO()
            print_exception(exc_info[0], exc_info[1], exc_info[2], None, out)
            success = False
            message = i18n("Unknown error")
            data = out.getvalue()
        self.setComplete(message, success, data)

    def runJob(self):
        '''
        The actual main job function which must be overriden by descendants
        @return the tuple (success, message, data) where:
        - success is set to True if the job completed successfully
        - message an optional message
        - data the optional data to return to the main thread
        '''
        pass

class ProgressDialog(KProgressDialog):
    """
    A KProcessDialog that can process custom events sent by a Job
    """
    def __init__(self, job, parent, name, caption, text, cancelButtonText, autoClose):
            """
            Constructor
            @param[in] job the job to run
            @param[in] parent parent of the progress dialog
            @param[in] name name of the progress dialog
            @param[in] caption text to display in window title bar
            @param[in] text text to display in the dialog
            @param[in] cancelButtonText text to show for the cancel button or
                None to disallow cancellation and to hide the cancel button
            @param[in] autoClose set to True, if the dialog should close itself on completion
            """
            KProgressDialog.__init__(self, parent, name, caption, text, True)
            self.job = job
            self.autoClose = autoClose
            self.results = [False, None]
            if cancelButtonText:
                self.setButtonText(cancelButtonText)
            else:
                self.setAllowCancel(False)
            self.connect(self, SIGNAL("cancelClicked()"), self.onCancel)
    def onCancel(self):
        '''
        Called when the user clicks on the cancel button
        '''
        if not self.job.isCancelled() and self.job.running():
            self.job.cancel()
            self.job.wait(CANCEL_TIMEOUT)
    def customEvent(self, event):
        """
        This is the event handler for custom events
        @param[in] event the QCustomEvent event
        """
        if event.type() == ProgressEvent.ID:
            if event.total != self.progressBar().totalSteps():
                self.progressBar().setTotalSteps(event.total)
            self.progressBar().setProgress(event.progress)
        elif event.type() == StatusEvent.ID:
            self.progressBar().setLabel(event.text)
        elif event.type() == TaskFinishedEvent.ID:
            self.results = [event.success, event]
            if self.autoClose:
                self.close()

def runJob(job, parent, caption, text, cancelButtonText = None, autoClose = False, name = None):
    """
    Launch a worker thread
    @param[in] job the job to run
    @param[in] parent parent of the progress dialog
    @param[in] caption text to display in window title bar
    @param[in] text text to display in the dialog
    @param[in] cancelButtonText text to show for the cancel button or
        None to disallow cancellation and to hide the cancel button
    @param[in] autoClose set to True, if the dialog should close itself on completion
    @param[in] name name of the progress dialog
    @return [success, event] where success is set to True if the job completed successfully and event to the resulting TaskFinishedEvent
    """
    dlg = ProgressDialog(job, parent, name, caption, text, cancelButtonText, autoClose)
    job.setDialog(dlg)
    job.start()
    dlg.exec_loop()
    return dlg.results
