#!/usr/bin/env python
# -*- coding: utf-8 -*-
#*****************************************************************************
#       Copyright (C) 2006 Serge Weinstock <serge.weinstock@ntlworld.com>
#  Distributed under the terms of the BSD License.  The full license is in
#  the file COPYING, distributed as part of this software.
#*****************************************************************************
"""
The application main frame
"""
import sys, imageview, collectionview, webshots, kdeutils
from qt import QString, SIGNAL, QCustomEvent, QCursor, QWidget
from kmdi import KMdiMainFrm, KMdi
from kdecore import KApplication, i18n, KURL, KURLDrag, KGlobal, KShortcut
from kdeui import KAction, KStdAction, KRadioAction, KKeyDialog, KMessageBox
from kio import KIO
from batchconversiondlgimpl import BatchConvertDlgImpl
from kdeutils import getCustomEventID

class ActivateViewEvent(QCustomEvent):
    '''
    An event for activating a view asynchronously
    '''
    ID = getCustomEventID()
    def __init__(self, view):
        '''
        Constructor
        @param[in] view the view to activate
        '''
        QCustomEvent.__init__(self, self.ID)
        self.view = view

class ViewClosedEvent(QCustomEvent):
    '''
    An event for managing the closing of a view asynchronously
    '''
    ID = getCustomEventID()
    def __init__(self, view):
        '''
        Constructor
        @param[in] view the view to activate
        '''
        QCustomEvent.__init__(self, self.ID)
        self.view = view

class RelayWidget(QWidget):
    """
    The sole purpose of this widget is to relay custom events to the parent window

    A bug in my version of PyKDE makes the application crash if we override the customEvent method
    of KMdiMainFrm.  So I've created an invisible widget for handling my custom events which will
    be processed by calling the appropriate method on the parent
    """
    def __init__(self, parent):
        """
        Constructor

        @param[in] parent the parent window to which to relay custom events
        """
        QWidget.__init__(self, parent, None, 0)
    def customEvent(self, event):
        """
        This is the event handler for custom events

        @param[in] event the QCustomEvent event
        """
        self.parent().customEventEx(event)


class MainFrame(KMdiMainFrm):
    """The application main frame"""

    def __init__(self, parent=None):
        """
        Constructor

        @param[in] parent the parent window
        """
        # need to read the mdi mode from the config before calling the base class constructor
        self.cfg = KApplication.kApplication().cfg.mainframe
        #
        KMdiMainFrm.__init__(self, parent, 'kUWC_main', self.cfg.MDImode)
        #
        self.ce = RelayWidget(self)
        self.views = []
        #
        self.setTabWidgetVisibility(KMdi.AlwaysShowTabs)
        self.setAcceptDrops(True)
        self.__initActions()
        self.setXMLFile("kUWCui.rc")
        self.createGUI(None)
        self.statusBar().show()
        #
        self.setAutoSaveSettings()
        #
        self.connect(self, SIGNAL('viewActivated(KMdiChildView *)'), self.childActivated)
    def __initActions(self):
        """
        Create all the application main actions
        """
        # "File" menu items
        self.openAction = KStdAction.open(self.open, self.actionCollection())
        self.openRecentFilesAction = KStdAction.openRecent(self.openURLBuggy, self.actionCollection())
        # the above connection doesn't work as it calls openURL without argument
        # we need to connect it again
        self.connect(self.openRecentFilesAction, SIGNAL("urlSelected(const KURL &)"), self.openURL)
        self.openRecentFilesAction.loadEntries(KGlobal.config(), "RecentFiles")
        self.openRecentFilesAction.dumpObjectInfo()
        self.quitAction = KStdAction.quit(self.close, self.actionCollection())
        self.closeAction = KStdAction.close(self.closeActiveChild, self.actionCollection())
        # "Settings" menu items
        self.showToolbarAction =  KStdAction.showToolbar(self.showToolbar, self.actionCollection())
        self.showStatusBarAction =  KStdAction.showStatusbar(self.showStatusBar, self.actionCollection())
        KStdAction.keyBindings(self.configureKeys, self.actionCollection());
        KStdAction.configureToolbars(self.configureToolbars, self.actionCollection())
        self.exportAction = KAction (i18n("&Save image"), "filesaveas", KShortcut.null(), self.save,
            self.actionCollection(), "save_image")
        self.exportAllAction = KAction (i18n("&Save all images"), "save_all", KShortcut.null(), self.saveAll,
            self.actionCollection(), "save_all_images")
        self.batchConvertAction = KAction (i18n("&Batch convert"), "launch", KShortcut.null(), self.batchConvert,
            self.actionCollection(), "batch_convert")
        # "View" menu items
        # NB: the FitToPage and ActualSize standard actions are redefined in order to behave as "radio actions"
        self.fitToPageAction = KRadioAction (i18n("&Fit to Page"), "viewmag1", KShortcut.null(), self.setZoom,
            self.actionCollection(), "img_fit_to_page")
        self.fitToPageAction.setExclusiveGroup ("Zoom")
        self.fitToPageAction.setChecked (1)
        self.actualSizeAction = KRadioAction (i18n("&Actual Size"), "view_fit_window", KShortcut.null(), self.setZoom,
            self.actionCollection(), "img_actual_size")
        self.actualSizeAction.setExclusiveGroup ("Zoom")
        # set their initial state
        self.__updateActions()

    def openURLBuggy(self):
        '''
        Dummy slot used as an argument to KStdAction.openRecent
        see comments in __initActions
        '''
        pass

    def save(self):
        '''
        Slot for saving an image
        '''
        view = self.activeWindow()
        if view:
            view.saveCurrentImage()

    def saveAll(self):
        '''
        Slot for saving all images
        '''
        view = self.activeWindow()
        if view:
            view.saveAllImages()

    def setZoom(self):
        '''
        Slot for the Stretch/Fit to page actions
        '''
        view = self.activeWindow()
        if view:
            view.stretchMode = (self.sender() == self.fitToPageAction)

    def showStatusBar(self):
        """
        This event handler is called when the "Show StatusBar" action is activated
        """
        self.statusBar().setShown(self.showStatusBarAction.isChecked())

    def showToolbar(self):
        """
        This event handler is called when the "Show Toolbar" action is activated
        """
        self.toolBar().setShown(self.showToolbarAction.isChecked())

    def configureKeys(self):
        """
        This event handler is called when the "configure Shortcuts" action is activated
        """
        KKeyDialog.configure(self.actionCollection())

    def configureToolbars(self):
        """
        This event handler is called when the "configure toolBars" action is activated
        """
        self.saveMainWindowSettings(KGlobal.config())

    def dragEnterEvent(self,event):
        """
        This event handler is called when a drag is in progress and the mouse enters this widget.
        @param[in] event the QDragEnterEvent event
        """
        # accept uri drops only
        event.accept(KURLDrag.canDecode(event));

    def dropEvent(self,event):
        """
        This event handler is called when the drag is dropped on this widget.
        @param[in] event the QDropEvent event
        """
        urls = KURL.List()
        # see if we can decode a URI.. if not, just ignore it
        if  KURLDrag.decode(event, urls) and (not urls.isEmpty()):
            # okay, we have some URIs.. let process them
            for url in urls:
                self.openURL(url)

    def open(self):
        """Show the open file dialog and open the selected file"""
        urls = KApplication.kApplication().selectFiles(self)
        if not urls:
            return
        for url in urls:
            self.openURL(url)

    def openURL(self, url):
        """
        Open a given URL

        @param[in] url the URL to open
        """
        KApplication.kApplication().setOverrideCursor(QCursor.waitCursor)
        view = None
        try:
            self.setDisabled(True)
            # already loaded ?
            for view in self.views:
                if view.url == url:
                    return
            view = None
            # if the file is not local. download it
            target = QString()
            if not KIO.NetAccess.download(url, target, self):
                KMessageBox.error(self, i18n("Unable to access the file:\n'%1'!").arg(url.prettyURL()))
                return
            # let's process it
            try:
                try:
                    f = open(unicode(target), 'rb')
                    try:
                        # let's try first a WB[1DZ] file
                        pixmap = webshots.WB1.read(url, f)
                        if pixmap:
                            view = imageview.ImageView(url, pixmap, self)
                        else:
                            # let's try first a WBC file
                            title, pixmaps = webshots.WBC.read(url, f)
                            if not pixmaps:
                                raise Exception(i18n("Invalid file type"))
                            view = collectionview.CollectionView(url, title, pixmaps, self)
                        self.views.append(view)
                        self.addWindow(view)
                        self.openRecentFilesAction.addURL(url)
                        self.setCaption(view.maincaption)
                        self.connect(view, SIGNAL('childWindowCloseRequest(KMdiChildView *)'), self.closeChild)
                        view.setDisabled(False)
                        self.cfg.lastOpenDir = url.upURL().prettyURL()
                    finally:
                        f.close()
                except:
                    error = sys.exc_info()
                    kdeutils.detailedError(self, i18n("Unable to read the file:\n'%1'").arg(url.prettyURL()), error)
            finally:
                # and remove the temp file if it's not a local file
                KIO.NetAccess.removeTempFile(target)
        finally:
            self.setDisabled(False)
            # strangely activateView doesn't work if just called after mainframe.setDisabled(True)
            # we need to do it asynchronously
            if view: KApplication.kApplication().postEvent(self.ce, ActivateViewEvent(view))
            KApplication.kApplication().restoreOverrideCursor()

    def childActivated(self, view):
        """
        Called when a view is activated

        @param[in] view the activated view
        """
        try:
            self.setCaption(view.maincaption)
            self.__updateActions()
        except:
            pass

    def closeActiveChild(self):
        """
        Close the current view
        """
        self.closeActiveView()

    def closeChild(self, view):
        """
        Called when a view is closed

        @param[in] view the closed view
        """
        self.removeWindowFromMdi(view)
        view.close()
        # it seems that there is a problem with reference counting:
        # if we remove here the view from self.views, the view C++ object seems to be deleted
        # and it leads to a crash
        # So we have to delay the update of self.views by posting an event that will be processed
        # after we get out of this event processing
        # (if only KMdiFrm.createIterator was implemented, we wouldn't need self.views...)
        KApplication.kApplication().postEvent(self.ce, ViewClosedEvent(view))

    def customEventEx(self, event):
        """
        This is the event handler for custom events

        A bug in my version of PyKDE makes the application crash if we override the customEvent method
        of KMdiMainFrm.  So I've created an invisible widget for handling my custom events which will
        be processed by calling this method

        @param[in] event the QCustomEvent event
        """
        if event.type() == ViewClosedEvent.ID:
            # now we can safely update self.views
            self.views.remove(event.view)
            if not self.views:
                self.__updateActions()
        elif event.type() == ActivateViewEvent.ID:
            # now activateView should work
            self.activateView(event.view)

    def queryClose(self):
        """
        Called before the window is closed, either by the user or indirectly by the session manager.

        @return True if the window can be closed
        """
        # save config
        self.openRecentFilesAction.saveEntries(KGlobal.config(), "RecentFiles")
        return True

    def __updateActions(self):
        """
        Update the actions according to the currently active view
        """
        view = self.activeWindow()
        if view:
            self.closeAction.setEnabled(True)
            self.fitToPageAction.setEnabled(True)
            self.fitToPageAction.setChecked(view.stretchMode)
            self.actualSizeAction.setEnabled(True)
            self.actualSizeAction.setChecked(not view.stretchMode)
            self.exportAction.setEnabled(True)
            self.exportAllAction.setEnabled(isinstance(view, collectionview.CollectionView))
        else:
            self.closeAction.setEnabled(False)
            self.fitToPageAction.setEnabled(False)
            self.actualSizeAction.setEnabled(False)
            self.exportAction.setEnabled(False)
            self.exportAllAction.setEnabled(False)

    def batchConvert(self):
        '''
        '''
        dlg = BatchConvertDlgImpl(self)
        dlg.exec_loop()
    def readProperties(self, cfg): #@UnusedVariable
        '''
        Read this instance-specific properties.
        (called when restoring the window from a saved session)
        @param cfg:
        '''
        # at the moment we don't do anything special
        pass

    def saveProperties(self, cfg): #@UnusedVariable
        '''
        Save this instance-specific properties.
        (called when saving a session)
        @param cfg:
        '''
        # at the moment we don't do anything special
        pass
